import 'package:provide/provide.dart';
import 'package:flutter/material.dart';
import 'dart:collection';

/*
 * @auther YuanShuai
 * @created at 2019/4/9 17:55
 * @desc BaseStateManage 状态管理
 */
class DBaseStateM extends ProviderNode {

  DBaseStateM({w,p}) : super(child: w, providers: p, dispose: true);
}

class DBaseN implements Listenable {
  ObserverList<VoidCallback> _listeners = ObserverList<VoidCallback>();

  bool _debugAssertNotDisposed() {
    assert(() {
      if (_listeners == null) {
        throw FlutterError(' $runtimeType 你如果执行了dispose()就不能在使用它了');
      }
      return true;
    }());
    return true;
  }

  bool get hasListeners {
    assert(_debugAssertNotDisposed());
    return _listeners.isNotEmpty;
  }

  void addListener(VoidCallback listener) {
    assert(_debugAssertNotDisposed());
    _listeners.add(listener);
  }

  void dispose() {
    assert(_debugAssertNotDisposed());
    _listeners = null;
  }

  void dNotifyChanges() {
    assert(_debugAssertNotDisposed());
    if (_listeners != null) {
      final List<VoidCallback> dlocalListeners =
          List<VoidCallback>.from(_listeners);
      for (VoidCallback listener in dlocalListeners) {
        try {
          if (_listeners.contains(listener)) listener();
        } catch (exception, stack) {
          FlutterError.reportError(FlutterErrorDetails(
              exception: exception,
              stack: stack,
              library: 'foundation library',
              context: '$runtimeType 在调度通知',
              informationCollector: (StringBuffer information) {
                information
                    .writeln('$runtimeType 当前发送通知的是:');
                information.write('  $this');
              }));
        }
      }
    }
  }

  void removeListener(listener) {
    assert(_debugAssertNotDisposed());
    _listeners.remove(listener);
  }
}

class ObserverList<T> extends Iterable<T> {
  final List<T> _list = <T>[];
  bool _isDirty = false;
  HashSet<T> _set;

  void add(T item) {
    _isDirty = true;
    _list.add(item);
  }

  bool remove(T item) {
    _isDirty = true;
    return _list.remove(item);
  }

  bool contains(Object element) {
    if (_list.length < 3) return _list.contains(element);

    if (_isDirty) {
      if (_set == null) {
        _set = HashSet<T>.from(_list);
      } else {
        _set.clear();
        _set.addAll(_list);
      }
      _isDirty = false;
    }

    return _set.contains(element);
  }

  Iterator<T> get iterator => _list.iterator;

  bool get isEmpty => _list.isEmpty;

  bool get isNotEmpty => _list.isNotEmpty;
}
